from typing import TypedDict


class SearchResult(TypedDict):
    title: str
    description: str
    link: str
    company: str
    location: str
    listed_date: str
    uid: str  # should have a custom id, usefull for other app projects to determine which has been seen already



from typing import Any, Dict

import requests
import undetected_chromedriver as uc
from bs4 import BeautifulSoup
from cloudscraper import create_scraper

default_headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"
}


def request(url: str, headers: Dict[str, Any] = {}) -> BeautifulSoup:
    r = requests.get(url, headers=default_headers | headers)

    if not r.ok:
        raise Exception("Request is not ok.")

    return BeautifulSoup(r.text, "lxml")


def c_request(url: str, headers: Dict[str, Any] = {}) -> BeautifulSoup:
    sess = requests.Session()
    sess.headers = default_headers | headers

    scraper = create_scraper(sess=sess)

    r = scraper.get(url)

    if not r.ok:
        raise Exception("Request is not ok.")

    return BeautifulSoup(r.text, "lxml")


def u_request(url: str) -> BeautifulSoup:
    options = uc.ChromeOptions()
    options.headless = True

    driver = uc.Chrome(
        options=options, version_main=110  # my own version of chrome (TODO: update)
    )

    driver.get(url)
    src = driver.page_source

    return BeautifulSoup(src, "lxml")



from typing import List
from urllib.parse import quote

from job_scraper.types import SearchResult
from job_scraper.utils import c_request

BASE_URL = "https://ph.jora.com"


class Jora:
    def __init__(self) -> None:
        pass

    def search(self, query: str, location: str, page: int = 1):
        url = BASE_URL + f"/j?q={quote(query)}&l={quote(location)}&st=date&p={page}"

        r = c_request(url)

        container = r.find("div", id="jobresults")
        all_results = container.find_all("article", class_="job-card")
        results: List[SearchResult] = []

        for i in all_results:
            raw_job_link = i.find("a", class_="job-link")

            job_title = raw_job_link.get_text().strip()
            job_id = i.get("id")
            job_link = BASE_URL + raw_job_link.get("href")
            job_company = i.find("span", class_="job-company").get_text().strip()
            job_location = i.find("a", class_="job-location").get("href")
            job_description = i.find("div", class_="job-abstract").get_text().strip()

            try:
                job_listed_date = (
                    i.find("span", class_="job-listed-date").get_text().strip()
                )
            except Exception:
                job_listed_date = ""

            results.append(
                {
                    "uid": job_id,
                    "title": job_title,
                    "company": job_company,
                    "description": job_description,
                    "link": job_link,
                    "location": job_location,
                    "listed_date": job_listed_date,
                }
            )

        return results
